//@ts-check
"use strict";

/**
@type {{
  pixiRoot: PIXI.Container;
  loop: PIXI.AnimatedSprite;
  title: PIXI.Text;

  loopTimer: number;
  loopForward: boolean;
}}
*/
let gameOver = null;

function registerGameOverResources() {
  app.loader.add([
    "gameOver/loop.png",
    "gameOver/loop.sprite.json"
  ]);
}

function enterGameOver() {
  ensureMenuMusic("gameOver");

  const pixiRoot = new PIXI.Container();
  game.screenPixiRoot.addChild(pixiRoot);

  const loopSpritesheet = getSpritesheet("gameOver/loop");
  const loop = new PIXI.AnimatedSprite(loopSpritesheet.animations["loop"]);
  loop.anchor.x = 0.5;
  loop.play();
  pixiRoot.addChild(loop);

  const title = new PIXI.Text(getText("gameOver.title"), new PIXI.TextStyle({
    fontFamily: "Arvo", fontSize: 192, fontWeight: "bold", align: "center",
    fill: "#fff", strokeThickness: 16, stroke: "#513a43", lineJoin: "bevel"
  }));
  title.anchor.set(0.5);
  title.scale.set(0.25);
  title.position.set(game.width / 2, game.height * 3 / 8);
  pixiRoot.addChild(title);

  const text = new PIXI.Text(getText("gameOver.text"), new PIXI.TextStyle({
    fontFamily: "Arvo", fontSize: 128, align: "center",
    fill: "#ffddb2", strokeThickness: 16, stroke: "#513a43", lineJoin: "bevel"
  }));
  text.anchor.set(0.5);
  text.scale.set(0.25);
  text.position.set(game.width / 2, game.height * 5 / 8);
  pixiRoot.addChild(text);

  gameOver = {
    pixiRoot,
    loop,
    loopTimer: 0,
    loopForward: true,
    title,
  };

  app.ticker.add(animateGameOver);
  document.addEventListener("keydown", onGameOverKeyDown);
  document.addEventListener("pointerdown", onGameOverPointerDown);
}

function leaveGameOver(enter) {
  document.removeEventListener("pointerdown", onGameOverPointerDown);
  document.removeEventListener("keydown", onGameOverKeyDown);
  app.ticker.remove(animateGameOver);

  gameOver.pixiRoot.destroy();
  gameOver = null;

  enter();
}

function onGameOverKeyDown(event) {
  switch (event.code) {
    case "Escape": leaveGameOver(enterMainMenu); break;
    case "Enter": leaveGameOver(enterInGame); break;
  }
}
function onGameOverPointerDown(event) {
  if (event.button !== 0) return;

  leaveGameOver(enterInGame);
}

function animateGameOver() {
  const dt = PIXI.Ticker.shared.elapsedMS / 1000;

  gameOver.loopTimer += 0.1 * (gameOver.loopForward ? dt : -dt);
  if (Math.abs(gameOver.loopTimer) > 1) {
    gameOver.loopTimer = Math.sign(gameOver.loopTimer);
    gameOver.loopForward = !gameOver.loopForward;
  }

  gameOver.loop.x = game.width / 2 + Math.sign(gameOver.loopTimer) * easeOutQuad(Math.abs(gameOver.loopTimer)) * 80;
}
